@@ -116,14 +116,6 @@ struct _GstCogScaleClass
 
 GType gst_cog_scale_get_type (void);
 
-
-/* elementfactory information */
-static const GstElementDetails cog_scale_details =
-GST_ELEMENT_DETAILS ("Video scaler",
-    "Filter/Effect/Video",
-    "Resizes video",
-    "Wim Taymans <wim.taymans@chello.be>");
-
 #define DEFAULT_QUALITY 5
 
 enum
@@ -222,7 +214,9 @@ gst_cog_scale_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &cog_scale_details);
+  gst_element_class_set_details_simple (element_class, "Video scaler",
+      "Filter/Effect/Video",
+      "Resizes video", "Wim Taymans <wim.taymans@chello.be>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_cog_scale_src_template));
