@@ -88,13 +88,6 @@ static AuBool NAS_EventHandler (AuServer * aud, AuEvent * ev,
 static AuDeviceID NAS_getDevice (AuServer * aud, int numTracks);
 static int NAS_createFlow (GstNasSink * sink, GstRingBufferSpec * spec);
 
-static const GstElementDetails nas_sink_details =
-GST_ELEMENT_DETAILS ("NAS audio sink",
-    "Sink/Audio",
-    "Plays audio to a Network Audio Server",
-    "Laurent Vivier <Laurent.Vivier@bull.net>, "
-    "Arwed v. Merkatz <v.merkatz@gmx.net>");
-
 GST_BOILERPLATE (GstNasSink, gst_nas_sink, GstAudioSink, GST_TYPE_AUDIO_SINK);
 
 static void
@@ -104,7 +97,11 @@ gst_nas_sink_base_init (gpointer g_class)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory));
-  gst_element_class_set_details (element_class, &nas_sink_details);
+  gst_element_class_set_details_simple (element_class, "NAS audio sink",
+      "Sink/Audio",
+      "Plays audio to a Network Audio Server",
+      "Laurent Vivier <Laurent.Vivier@bull.net>, "
+      "Arwed v. Merkatz <v.merkatz@gmx.net>");
 }
 
 static void
@@ -171,12 +168,6 @@ gst_nas_sink_getcaps (GstBaseSink * bsink)
 
   server = nassink->audio;
 
-  if (server == NULL) {
-    /* FIXME: is it really a good idea to do a potentially long blocking call
-     * like this here? (tpm) */
-    server = AuOpenServer (nassink->host, 0, NULL, 0, NULL, NULL);
-  }
-
   templatecaps = gst_static_pad_template_get_caps (&sink_factory);
 
   if (server == NULL)
@@ -293,9 +284,9 @@ gst_nas_sink_set_property (GObject * object, guint prop_id,
       g_free (nassink->host);
       nassink->host = g_value_dup_string (value);
       if (nassink->host == NULL)
-        nassink->host = g_strdup (getenv ("AUDIOSERVER"));
+        nassink->host = g_strdup (g_getenv ("AUDIOSERVER"));
       if (nassink->host == NULL)
-        nassink->host = g_strdup (getenv ("DISPLAY"));
+        nassink->host = g_strdup (g_getenv ("DISPLAY"));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -333,8 +324,10 @@ gst_nas_sink_open (GstAudioSink * asink)
 
   /* Open Server */
   sink->audio = AuOpenServer (sink->host, 0, NULL, 0, NULL, NULL);
-  if (sink->audio == NULL)
+  if (sink->audio == NULL) {
+    GST_DEBUG_OBJECT (sink, "opening failed");
     return FALSE;
+  }
   sink->flow = AuNone;
   sink->need_data = 0;
 
