@@ -17,8 +17,10 @@
  *
  */
 
-#define FLOAT_SAMPLES 1
-#include <soundtouch/SoundTouch.h>
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 /* FIXME: workaround for SoundTouch.h of version 1.3.1 defining those
  * variables while it shouldn't. */
 #undef VERSION
@@ -29,9 +31,8 @@
 #undef PACKAGE_BUGREPORT
 #undef PACKAGE
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#define FLOAT_SAMPLES 1
+#include <soundtouch/SoundTouch.h>
 
 #include <gst/gst.h>
 #include <gst/controller/gstcontroller.h>
@@ -181,8 +182,6 @@ gst_pitch_init (GstPitch * pitch, GstPitchClass * pitch_class)
       GST_DEBUG_FUNCPTR (gst_pad_proxy_getcaps));
   gst_element_add_pad (GST_ELEMENT (pitch), pitch->srcpad);
 
-  gst_element_no_more_pads (GST_ELEMENT (pitch));
-
   pitch->priv->st = new soundtouch::SoundTouch ();
 
   pitch->tempo = 1.0;
@@ -668,10 +667,6 @@ gst_pitch_src_query (GstPad * pad, GstQuery * query)
           GST_DEBUG ("Calculated total latency : min %"
               GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
               GST_TIME_ARGS (min), GST_TIME_ARGS (max));
-          g_print ("Calculated total latency : min %"
-              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
-              GST_TIME_ARGS (min), GST_TIME_ARGS (max));
-
           gst_query_set_latency (query, live, min, max);
         }
         gst_object_unref (peer);
