@@ -23,6 +23,8 @@
 
 #include <gst/gstbin.h>
 
+#include "gstcamerabin-enum.h"
+
 G_BEGIN_DECLS
 #define GST_TYPE_CAMERABIN_IMAGE             (gst_camerabin_image_get_type())
 #define GST_CAMERABIN_IMAGE_CAST(obj)        ((GstCameraBinImage*)(obj))
@@ -48,11 +50,12 @@ struct _GstCameraBinImage
 
   GstElement *post;
   GstElement *enc;
-  GstElement *user_enc;
+  GstElement *app_enc;
   GstElement *meta_mux;
   GstElement *sink;
 
   gboolean elements_created;
+  GstCameraBinFlags flags;
 };
 
 struct _GstCameraBinImageClass
@@ -69,9 +72,13 @@ void
 gst_camerabin_image_set_postproc (GstCameraBinImage * img,
     GstElement * postproc);
 
+void
+gst_camerabin_image_set_flags (GstCameraBinImage * img,
+    GstCameraBinFlags flags);
+
 GstElement *gst_camerabin_image_get_encoder (GstCameraBinImage * img);
 
 GstElement *gst_camerabin_image_get_postproc (GstCameraBinImage * img);
 
 G_END_DECLS
-#endif                          /* #ifndef __CAMERABIN_IMAGE_H__ */
+#endif /* #ifndef __CAMERABIN_IMAGE_H__ */
