@@ -83,12 +83,6 @@ enum
 GST_DEBUG_CATEGORY_STATIC (video_mark_debug);
 #define GST_CAT_DEFAULT video_mark_debug
 
-static const GstElementDetails video_mark_details =
-GST_ELEMENT_DETAILS ("Video marker",
-    "Filter/Effect/Video",
-    "Marks a video signal with a pattern",
-    "Wim Taymans <wim@fluendo.com>");
-
 static GstStaticPadTemplate gst_video_mark_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -327,7 +321,9 @@ gst_video_mark_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &video_mark_details);
+  gst_element_class_set_details_simple (element_class, "Video marker",
+      "Filter/Effect/Video",
+      "Marks a video signal with a pattern", "Wim Taymans <wim@fluendo.com>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_video_mark_sink_template));
