@@ -28,13 +28,7 @@
 
 #include "windows.h"
 
-#define WM_GRAPH_NOTIFY WM_APP + 1 /* Private message */
-
-static const GstElementDetails gst_dshowvideosink_details =
-GST_ELEMENT_DETAILS ("DirectShow video sink",
-    "Sink/Video",
-    "Display data using a DirectShow video renderer",
-    "Pioneers of the Inevitable <songbird@songbirdnest.com>");
+#define WM_GRAPH_NOTIFY WM_APP + 1
 
 GST_DEBUG_CATEGORY (dshowvideosink_debug);
 #define GST_CAT_DEFAULT dshowvideosink_debug
@@ -145,7 +139,10 @@ gst_dshowvideosink_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_template));
 
-  gst_element_class_set_details (element_class, &gst_dshowvideosink_details);
+  gst_element_class_set_details_simple (element_class, "DirectShow video sink",
+      "Sink/Video",
+      "Display data using a DirectShow video renderer",
+      "Pioneers of the Inevitable <songbird@songbirdnest.com>");
 }
 
 static void
@@ -405,7 +402,6 @@ gst_dshowvideosink_handle_event (GstDshowVideoSink *sink)
   if (sink->filter_media_event) {
     long evCode;
     LONG_PTR param1, param2;
-    HRESULT hr;
     while (SUCCEEDED (sink->filter_media_event->GetEvent(&evCode, &param1, &param2, 0)))
     {
       GST_INFO_OBJECT (sink, "Received DirectShow graph event code 0x%x", evCode);
@@ -522,8 +518,15 @@ gst_dshowvideosink_window_thread (GstDshowVideoSink * sink)
      * of the video.
      */
     RECT rect;
-    VIDEOINFOHEADER *vi = (VIDEOINFOHEADER *)sink->mediatype.pbFormat;
+    AM_MEDIA_TYPE pmt = (AM_MEDIA_TYPE)sink->mediatype;
+    VIDEOINFOHEADER *vi = (VIDEOINFOHEADER *)pmt.pbFormat;
 
+    if (vi == NULL)
+    {
+        GST_ELEMENT_ERROR (sink, RESOURCE, NOT_FOUND, ("Unknown media format"), (NULL));
+        return NULL;
+    }
+	
     /* rcTarget is the aspect-ratio-corrected size of the video. */
     width = vi->rcTarget.right + GetSystemMetrics (SM_CXSIZEFRAME) * 2;
     height = vi->rcTarget.bottom + GetSystemMetrics (SM_CYCAPTION) +
@@ -1351,6 +1354,11 @@ gst_dshowvideosink_stop (GstBaseSink * bsink)
     sink->filter_graph = NULL;
   }
 
+  if (sink->filter_media_event) {
+    sink->filter_media_event->Release();
+    sink->filter_media_event = NULL;
+  }
+
   return TRUE;
 }
 
